// gl_texloader.cpp by CrazyRussian 

#include "hud.h"
#include "cl_util.h"
#include "com_model.h"

#include <windows.h>
#include <gl/gl.h>
#include <gl/glext.h>

#pragma comment(lib,"Opengl32.lib")
#pragma warning(disable:4018)

#pragma pack(push,1)
typedef struct
{
	
    byte  identsize;          // size of ID field that follows 18 byte header (0 usually)
    byte  colourmaptype;      // type of colour map 0=none, 1=has palette
    byte  imagetype;          // type of image 0=none,1=indexed,2=rgb,3=grey,+8=rle packed
	
    short colourmapstart;     // first colour map entry in palette
    short colourmaplength;    // number of colours in palette
    byte  colourmapbits;      // number of bits per palette entry 15,16,24,32
	
    short xstart;             // image x origin
    short ystart;             // image y origin
    short width;              // image width in pixels
    short height;             // image height in pixels
    byte  bits;               // image bits per pixel 8,16,24,32
    byte  descriptor;         // image descriptor bits (vh flip bits)
    
    
    
} TGA_HEADER;
#pragma pack(pop)

int glLoadTgaFromPak(char* name,int dest)
{
	//      :
	int len;
	byte*buff=gEngfuncs.COM_LoadFile(name,5,&len); //  

	
	int width,height; //  
	TGA_HEADER hdr; //  
	
	if (!buff) //  =0    
	{
		return 0;
	}
	//     
	memcpy(&hdr,&buff[0],18);
	
	width=hdr.width;
	height=hdr.height;
	
	//     24  
	int type=GL_RGB;
	int bpp;
	//  
	switch(hdr.bits)
	{
	case 24:
		bpp=3;
		break;
	case 32:
		type=GL_RGBA;
		bpp=4;
		break;
	default:
		gEngfuncs.Con_Printf("glLoadTga(): file %s has unsupported bpp num: %d !\n",hdr.bits);
		return -1;
	}
	int imgsize=width*height*bpp;
	byte temp;

	//  BGR  RGB
	for(int i=18; i<int(imgsize); i+=bpp) 
	{                                                    
		temp=buff[i];                       
		buff[i]=buff[i+2];
		buff[i+2] = temp;                 
	}

	byte*oldbuff=(byte*)malloc(imgsize);

	memcpy(oldbuff,&buff[18],imgsize);

	int dest_offset=18;
	
	for(i=height-1;i>=0;i--)
	{
		byte*src_line=&oldbuff[width*bpp*i];
		
		memcpy(&buff[dest_offset],src_line,width*bpp);
		dest_offset+=width*bpp;
	}

	free(oldbuff);

	static unsigned int iRet;

	
	glBindTexture(GL_TEXTURE_2D, dest); //   
	//   
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); 
	glHint( GL_GENERATE_MIPMAP_HINT_SGIS, GL_NICEST );
	glTexParameteri( GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS, GL_TRUE );
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, type,width,height,0, type, GL_UNSIGNED_BYTE, &buff[18]);
	
	gEngfuncs.COM_FreeFile(buff);

	return dest;
	
}

bool g_tex_loaded=false;

void UploadWorldHDTextures()
{
	if (g_tex_loaded) return;
	g_tex_loaded=true;

	model_t*world=gEngfuncs.GetEntityByIndex(0)->model;

	for(int i=0;i<world->numtextures;i++)
	{
		char file_path[260];

		sprintf(file_path,"gfx/textures/%s.tga",world->textures[i]->name);

		glLoadTgaFromPak(file_path,world->textures[i]->gl_texturenum);
	}

}


//================
// HD Skybox
//================

/*
extern void WriteAtFile(char*path,char*data);



char *buildTgaName(int num)
{
char sky[32];
sprintf(sky,"%s",CVAR_GET_STRING("sv_skyname"));
switch(num)
{
case 5800:
strcat(sky,"dn.tga");
break;
case 5801:
strcat(sky,"up.tga");
break;
case 5802:
strcat(sky,"lf.tga");
break;
case 5803:
strcat(sky,"ft.tga");
break;
case 5804:
strcat(sky,"bk.tga");
break;
case 5805:
strcat(sky,"rt.tga");
break;
}
return 0;
}
*/
//DrawConsoleString( XRES(0), YRES(260), sky);
/*
return sky;
}
bool HackSky=false;
TextureImage hack[21];
extern void __CmdFunc_reload_skybox();
void HDSkyHack()
{


	if (CVAR_GET_FLOAT("r_hd_encache")==0 && IsWeakPC()) return;
	for (int hi=0;hi<6;hi++)
	{
		char sky[64];

sprintf(sky,"%s",CVAR_GET_STRING("sv_skyname"));
		hack[hi].texID=5800+hi;
			char temp[128];
	//sprintf(temp,"gfx//hd//desertup.tga",buildTgaName(5800+hi));

switch(5800+hi)
{
case 5800:
strcat(sky,"rt.tga");
break;
case 5801:
strcat(sky,"ft.tga");
break;
case 5802:
strcat(sky,"lf.tga");
break;
case 5803:
strcat(sky,"bk.tga");
break;
case 5804:
strcat(sky,"up.tga");
break;
case 5805:
strcat(sky,"dn.tga");
break;
}
sprintf(temp,"gfx//hd//sky//%s",sky);

	//WriteAtFile("hack.log",temp);
if (!hack[hi].Loaded )	 
{
	if (hack[hi].NotFoundSoDontTryToLoadAnymore==false)
	{
if 	(!glLoadTGA(&hack[hi],temp)) hack[hi].NotFoundSoDontTryToLoadAnymore=true ;

	}

}
else 
	  {
		if (HackSky) return;
			glBindTexture(GL_TEXTURE_2D, hack[hi].texID);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, hack[hi].width, hack[hi].height,                   
				
				0, GL_RGB, GL_UNSIGNED_BYTE, hack[hi].imageData);
			HackSky=true;
		}
	}
	
	
}

TextureImage WorldHack[512];
void FixSkys()
{
		for (int hi=0;hi<6;hi++)
	{
			glBindTexture(GL_TEXTURE_2D, hack[hi].texID);
			glTexSubImage2D(GL_TEXTURE_2D, 0, GL_RGB, hack[hi].width, hack[hi].height,                   
				
				0, GL_RGB, GL_UNSIGNED_BYTE, hack[hi].imageData);
		}
}
*/